// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.
package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"strconv"
)

func sortMerchantConsumeDetailInfo(array []apis.ApiServiceConsumeDetail) []apis.ApiServiceConsumeDetail {
	length := len(array)
	for i := 0; i < length; i++ {
		for j := i + 1; j < length; j++ {
			if array[i].Charge < array[j].Charge {
				tmp := array[i]
				array[i] = array[j]
				array[j] = tmp
			}
		}
	}
	return array
}

func sortMerchantConsumeInfo(array []apis.ApiServiceConsumeInfo) []apis.ApiServiceConsumeInfo {
	length := len(array)
	for i := 0; i < length; i++ {
		for j := i + 1; j < length; j++ {
			if array[i].Charge < array[j].Charge {
				tmp := array[i]
				array[i] = array[j]
				array[j] = tmp
			}
		}
	}
	for i, v := range array {
		array[i].Detail = sortMerchantConsumeDetailInfo(v.Detail)
	}
	return array
}

func apiServiceGetConsumeInfo(ctx context.Context, req *apis.BiGetApiServiceConsumeInfoReq, start, end int64, isToday bool) ([]apis.ApiServiceConsumeInfo, error) {
	mreq := apis.BiGetReportReq{}
	mreply := apis.BiGetReportReply{}
	mreq.IsAll = true
	mreq.TodayEndTimestamp = req.TodayEndTimestamp
	mreq.Date = getDatesFromTimestamp(start, end)
	var err error
	if isToday {
		mreply.Infos, err = getTodayReportFromHour(&mreq, start, end)
		if err != nil {
			return nil, err
		}
	} else {
		err = BiGetReport(ctx, &mreq, &mreply)
		if err != nil {
			return nil, err
		}
	}

	m := map[string]map[string]*apis.BiReportInfo{}
	for i, v := range mreply.Infos {
		if _, ok := m[v.MerchantName]; ok == false {
			item := map[string]*apis.BiReportInfo{}
			item[v.ApiName] = &mreply.Infos[i]
			m[v.MerchantName] = item
			continue
		}
		if _, ok := m[v.MerchantName][v.ApiName]; ok == false {
			m[v.MerchantName][v.ApiName] = &mreply.Infos[i]
			continue
		}
		m[v.MerchantName][v.ApiName].Charge += v.Charge
		m[v.MerchantName][v.ApiName].Valid += v.Valid
		m[v.MerchantName][v.ApiName].Total += v.Total
	}
	ret := []apis.ApiServiceConsumeInfo{}
	for merchantName, apiMap := range m {
		item := apis.ApiServiceConsumeInfo{}
		item.MerchantName = merchantName
		for _, api := range apiMap {
			apiItem := apis.ApiServiceConsumeDetail{}
			item.Charge += api.Charge
			apiItem.Charge = api.Charge
			apiItem.Valid = api.Valid
			apiItem.ApiName = api.ApiName
			apiItem.ChargeRate = strconv.FormatFloat(float64(100*api.Charge/api.Total), 'f', 2, 64) + "%"
			item.Detail = append(item.Detail, apiItem)
		}
		ret = append(ret, item)
	}
	for i, v := range ret {
		for k, d := range ret[i].Detail {
			if v.Charge == 0 {
				ret[i].Detail[k].Rate = "0%"
			} else {
				ret[i].Detail[k].Rate = strconv.FormatFloat(float64(100*d.Charge/v.Charge), 'f', 2, 64) + "%"
			}
		}
	}
	return ret, nil
}

func BiGetApiServiceConsumeInfo(ctx context.Context, req *apis.BiGetApiServiceConsumeInfoReq, reply *apis.BiGetApiServiceConsumeInfoReply) (err error) {
	// 参数预处理
	if req.PageNumber <= 0 {
		req.PageNumber = 1
	}
	if req.PageSize <= 0 {
		req.PageSize = pageSize
	}
	reply.PageNumber = req.PageNumber
	reply.PageSize = req.PageSize
	offset := (req.PageNumber - 1) * req.PageSize
	limit := req.PageSize
	start := int64(0)
	end := int64(0)

	// 今日，昨日，本周，本月
	isToday := false
	switch req.TimeType {
	case 0:
		isToday = true
		start, end = getTodayTimestamp()
	case 1:
		start, end = getYesterdayTimestamp()
	case 2:
		start, end = getWeekTimestamp()
	case 3:
		start, end = getMonthTimestamp()
	default:
		return errors.ArgsError
	}

	// 获取所有信息
	infos, err := apiServiceGetConsumeInfo(ctx, req, start, end, isToday)
	if err != nil {
		return err
	}
	reply.Total = int64(len(infos))
	infos = sortMerchantConsumeInfo(infos)
	reply.Infos = infos

	// 分页
	switch {
	case reply.Total-int64(offset) <= 0:
		reply.Infos = []apis.ApiServiceConsumeInfo{}
	case reply.Total-int64(offset) <= int64(limit):
		reply.Infos = reply.Infos[offset:]
	case reply.Total-int64(offset) > int64(limit):
		reply.Infos = reply.Infos[offset : offset+limit]
	}
	return nil
}
