// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package access_log

import (
	"context"
	"gd_statistics/apis"
	"time"
)

func getYesterdayTimestamp() (int64, int64) {
	now := time.Now()
	yes := now.AddDate(0, 0, -1)
	end := time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location()).Unix()
	start := time.Date(yes.Year(), yes.Month(), yes.Day(), 0, 0, 0, 0, yes.Location()).Unix()
	return start, end
}

func getWeekTimestamp() (int64, int64) {
	now := time.Now()
	t := now
	for {
		if t.Weekday() == 1 {
			break
		}
		t = t.AddDate(0, 0, -1)
	}
	start := time.Date(t.Year(), t.Month(), t.Day(), 0, 0, 0, 0, t.Location()).Unix()
	return start, now.Unix()
}

func getMonthTimestamp() (int64, int64) {
	now := time.Now()
	t := now
	for {
		if _, _, day := t.Date(); day == 1 {
			break
		}
		t = t.AddDate(0, 0, -1)
	}
	start := time.Date(t.Year(), t.Month(), t.Day(), 0, 0, 0, 0, t.Location()).Unix()
	return start, now.Unix()
}

func getDatesFromTimestamp(start, end int64) (ret []string) {
	for i := start; i < end; i += 24 * 3600 {
		date := time.Unix(i, 0).Format("2006-01-02")
		ret = append(ret, date)
	}
	return ret
}

func apiServiceGetDataInfo(ctx context.Context, req *apis.BiGetApiServiceDataInfoReq, start, end int64, isToday bool) (*apis.ApiServiceDataInfo, error) {
	mreq := apis.BiGetReportReq{}
	mreply := apis.BiGetReportReply{}
	if req.ApiId > 0 {
		mreq.ApiId = []int64{req.ApiId}
	}
	mreq.IsAll = true
	mreq.TodayEndTimestamp = req.TodayEndTimestamp
	mreq.Date = getDatesFromTimestamp(start, end)
	var err error
	if isToday {
		mreply.Infos, err = getTodayReportFromHour(&mreq, start, end)
		if err != nil {
			return nil, err
		}
	} else {
		err = BiGetReport(ctx, &mreq, &mreply)
		if err != nil {
			return nil, err
		}
	}
	ret := &apis.ApiServiceDataInfo{}
	for _, v := range mreply.Infos {
		ret.Valid += v.Valid
		ret.Charge += v.Charge
		ret.Reuse += v.Reuse
		ret.Fail += (v.Total - v.Success)
	}
	return ret, nil
}

func BiGetApiServiceDataInfo(ctx context.Context, req *apis.BiGetApiServiceDataInfoReq, reply *apis.BiGetApiServiceDataInfoReply) (err error) {
	start, end := getTodayTimestamp()
	todayStart := start
	info, err := apiServiceGetDataInfo(ctx, req, start, end, true)
	if err != nil {
		return err
	}
	reply.Info.Today = *info
	start, end = getYesterdayTimestamp()
	info, err = apiServiceGetDataInfo(ctx, req, start, end, false)
	if err != nil {
		return err
	}
	reply.Info.Yesterday = *info
	start, end = getWeekTimestamp()
	end = todayStart
	if start == end {
		info = &apis.ApiServiceDataInfo{}
	} else if info, err = apiServiceGetDataInfo(ctx, req, start, end, false); err != nil {
		return err
	}
	info.Charge += reply.Info.Today.Charge
	info.Valid += reply.Info.Today.Valid
	info.Reuse += reply.Info.Today.Reuse
	info.Fail += reply.Info.Today.Fail
	reply.Info.Week = *info

	start, end = getMonthTimestamp()
	end = todayStart
	if start == end {
		info = &apis.ApiServiceDataInfo{}
	} else if info, err = apiServiceGetDataInfo(ctx, req, start, end, false); err != nil {
		return err
	}
	info.Charge += reply.Info.Today.Charge
	info.Valid += reply.Info.Today.Valid
	info.Reuse += reply.Info.Today.Reuse
	info.Fail += reply.Info.Today.Fail
	reply.Info.Month = *info
	return nil
}
