// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.
package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"fmt"
	"strconv"

	"github.com/astaxie/beego/orm"
)

type TGdApiCodeHour struct {
	Id           int64  `json:"id"`
	MerchantId   int64  `json:"merchant_id"`
	ApiId        int64  `json:"api_id"`
	Code         int    `json:"code"`
	Msg          string `json:"msg"`
	Count        int64  `json:"count"`
	Hour         int64  `json:"hour"`
	State        bool   `json:"state"`
	MerchantName string `json:"merchant_name"`
	ApiName      string `json:"api_name"`
}

type TGdProviderCodeHour struct {
	Id              int64  `json:"id"`
	ProviderApiId   int64  `json:"api_id"`
	Code            string `json:"code"`
	Msg             string `json:"msg"`
	Count           int64  `json:"count"`
	Hour            int64  `json:"hour"`
	State           bool   `json:"state"`
	ProviderName    string `json:"provider_name"`
	ProviderApiName string `json:"provider_api_name"`
}

type TGdReportHour struct {
	Id                int64
	MerchantId        int64
	ApiId             int64
	MerchantName      string
	Date              string `json:"date"`
	MerchantDataApiId int64
	ApiName           string  `json:"api_name"`
	Total             int64   `json:"total"`
	Valid             int64   `json:"valid"`
	Success           int64   `json:"success"`
	Query             int64   `json:"query"`
	Reuse             int64   `json:"reuse"`
	SumElapsed        float64 `json:"sum_elapsed"`
	PlatformError     int64   `json:"platform_error"`
	ProviderError     int64   `json:"provider_error"`
	Charge            int64   `json:"charge"`
	Hour              int64   `json:"hour"`
}

type TGdProviderReportHour struct {
	Id              int64
	Date            string  `json:"date"`
	ProviderName    string  `json:"provider_name"`
	MerchantId      int64   `json:"merchant_id"`
	ApiId           int64   `json:"api_id"`
	ProviderApiName string  `json:"provider_api_name"`
	Total           int64   `json:"total"`
	Success         int64   `json:"success"`
	Failed          int64   `json:"failed"`
	Query           int64   `json:"query"`
	SumElapsed      float64 `json:"sum_elapsed"`
	Charge          int64   `json:"charge"`
	ProviderApiId   int64   `json:"provider_api_id"`
	Hour            int64   `json:"hour"`
}

type ApiCodeInfoReq struct {
	Start      int64
	End        int64
	MerchantId int64
	ApiId      int64
	OnlyFail   bool
	GroupByApi bool
}

func getApiCodeInfoFromHour(o orm.Ormer, req *ApiCodeInfoReq) ([]TGdApiCodeHour, error) {
	if req.End <= req.Start {
		return nil, nil
	}
	sql := fmt.Sprintf("select sum(count) as count, code, msg from t_gd_api_code_hour where hour >= %d and hour < %d", req.Start, req.End)
	if req.GroupByApi {
		sql = fmt.Sprintf("select sum(count) as count, code, msg, api_id, api_name from t_gd_api_code_hour where hour >= %d and hour < %d", req.Start, req.End)
	}
	if req.ApiId > 0 {
		sql = fmt.Sprintf("%s and api_id=%d", sql, req.ApiId)
	}
	if req.MerchantId > 0 {
		sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
	}
	if req.OnlyFail == true {
		sql = fmt.Sprintf("%s and state = 0", sql)
	}
	if req.GroupByApi {
		sql = fmt.Sprintf("%s group by code, msg, api_id", sql)
	} else {
		sql = fmt.Sprintf("%s group by code, msg", sql)
	}
	tabs := []TGdApiCodeHour{}
	_, err := o.Raw(sql).QueryRows(&tabs)
	if err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}
	return tabs, nil

}

func getApiCodeInfoFromToday(o orm.Ormer, req *ApiCodeInfoReq) ([]TGdApiCodeHour, error) {
	sql := fmt.Sprintf("select count(1) as count, raw_code as code, msg from t_gd_access_log_day where timestamp >= %d and timestamp < %d", req.Start, req.End)
	if req.GroupByApi {
		sql = fmt.Sprintf("select count(1) as count, raw_code as code, msg, api_id, api_name from t_gd_access_log_day where timestamp >= %d and timestamp < %d", req.Start, req.End)
	}
	if req.ApiId > 0 {
		sql = fmt.Sprintf("%s and api_id=%d", sql, req.ApiId)
	}
	if req.MerchantId > 0 {
		sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
	}
	if req.OnlyFail == true {
		sql = fmt.Sprintf("%s and state = 0", sql)
	}
	if req.GroupByApi {
		sql = fmt.Sprintf("%s group by code, msg, api_id", sql)
	} else {
		sql = fmt.Sprintf("%s group by code, msg", sql)
	}
	tabs := []TGdApiCodeHour{}
	_, err := o.Raw(sql).QueryRows(&tabs)
	if err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}
	return tabs, nil
}

func sumApiCode(old []TGdApiCodeHour, today []TGdApiCodeHour, groupByApi bool) (ret []TGdApiCodeHour) {
	m := map[string]*TGdApiCodeHour{}
	for i, v := range old {
		key := fmt.Sprintf("%d-%s", v.Code, v.Msg)
		if groupByApi {
			key = fmt.Sprintf("%d-%s-%d", v.Code, v.Msg, v.ApiId)
		}
		m[key] = &old[i]
	}
	for i, v := range today {
		key := fmt.Sprintf("%d-%s", v.Code, v.Msg)
		if groupByApi {
			key = fmt.Sprintf("%d-%s-%d", v.Code, v.Msg, v.ApiId)
		}
		if _, ok := m[key]; ok == false {
			m[key] = &today[i]
			continue
		}
		m[key].Count += v.Count
	}
	ret = make([]TGdApiCodeHour, len(m))
	i := 0
	for _, v := range m {
		ret[i] = *v
		i++
	}
	return ret
}

func getApiCodeInfo(req *ApiCodeInfoReq) ([]TGdApiCodeHour, error) {
	o := orm.NewOrm()
	maxHour := int64(0)
	err := o.Raw("select max(hour) as max_hour from t_gd_api_code_hour").QueryRow(&maxHour)
	if err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}
	todayInfos := []TGdApiCodeHour{}
	if maxHour+3600 < req.End {
		/*
			mreq := *req
			mreq.Start = maxHour + 3600
			todayInfos, err = getApiCodeInfoFromToday(o, &mreq)
			if err != nil {
				return nil, err
			}
		*/
	}
	mreq := *req
	mreq.End = maxHour + 3600
	oldInfos, err := getApiCodeInfoFromHour(o, &mreq)
	if err != nil {
		return nil, err
	}
	if len(todayInfos) == 0 {
		return oldInfos, nil
	}
	if len(oldInfos) == 0 {
		return todayInfos, nil
	}

	return sumApiCode(oldInfos, todayInfos, req.GroupByApi), nil
}

func getApiServiceFailInfo(start, end int64) ([]apis.ApiServiceFailInfo, error) {
	mreq := ApiCodeInfoReq{}
	mreq.GroupByApi = true
	mreq.Start = start
	mreq.End = end
	mreq.OnlyFail = true
	codeInfos, err := getApiCodeInfo(&mreq)
	if err != nil {
		return nil, err
	}
	m := map[int64][]*TGdApiCodeHour{}
	for i, v := range codeInfos {
		m[v.ApiId] = append(m[v.ApiId], &codeInfos[i])
	}
	ret := []apis.ApiServiceFailInfo{}
	for _, v := range m {
		item := apis.ApiServiceFailInfo{}
		for _, sub := range v {
			failItem := apis.ApiServiceFailDetail{}
			item.ApiName = sub.ApiName
			item.Fail += sub.Count
			failItem.Fail = sub.Count
			failItem.Msg = sub.Msg
			failItem.Code = sub.Code
			item.Detail = append(item.Detail, failItem)
		}
		for i, d := range item.Detail {
			if item.Fail == 0 {
				item.Detail[i].Rate = "0%"
			} else {
				item.Detail[i].Rate = strconv.FormatFloat(float64(100*d.Fail/item.Fail), 'f', 2, 64) + "%"
			}
		}
		ret = append(ret, item)
	}
	return ret, nil
}

func sortApiFailDetailInfo(array []apis.ApiServiceFailDetail) []apis.ApiServiceFailDetail {
	length := len(array)
	for i := 0; i < length; i++ {
		for j := i + 1; j < length; j++ {
			if array[i].Fail < array[j].Fail {
				tmp := array[i]
				array[i] = array[j]
				array[j] = tmp
			}
		}
	}
	return array
}

func sortApiFailInfo(array []apis.ApiServiceFailInfo) []apis.ApiServiceFailInfo {
	length := len(array)
	for i := 0; i < length; i++ {
		for j := i + 1; j < length; j++ {
			if array[i].Fail < array[j].Fail {
				tmp := array[i]
				array[i] = array[j]
				array[j] = tmp
			}
		}
	}
	for i, v := range array {
		array[i].Detail = sortApiFailDetailInfo(v.Detail)
	}
	return array
}

func BiGetApiServiceFailInfo(ctx context.Context, req *apis.BiGetApiServiceFailInfoReq, reply *apis.BiGetApiServiceFailInfoReply) (err error) {
	// 参数预处理
	if req.PageNumber <= 0 {
		req.PageNumber = 1
	}
	if req.PageSize <= 0 {
		req.PageSize = pageSize
	}
	reply.PageNumber = req.PageNumber
	reply.PageSize = req.PageSize
	offset := (req.PageNumber - 1) * req.PageSize
	limit := req.PageSize
	start := int64(0)
	end := int64(0)

	// 今日，昨日，本周，本月
	switch req.TimeType {
	case 0:
		start, end = getTodayTimestamp()
	case 1:
		start, end = getYesterdayTimestamp()
	case 2:
		start, end = getWeekTimestamp()
	case 3:
		start, end = getMonthTimestamp()
	default:
		return errors.ArgsError
	}

	if req.TodayEndTimestamp < end && req.TodayEndTimestamp > 0 {
		end = req.TodayEndTimestamp
	}

	infos, err := getApiServiceFailInfo(start, end)
	if err != nil {
		return err
	}

	reply.Total = int64(len(infos))
	infos = sortApiFailInfo(infos)
	reply.Infos = infos

	// 分页
	switch {
	case reply.Total-int64(offset) <= 0:
		reply.Infos = []apis.ApiServiceFailInfo{}
	case reply.Total-int64(offset) <= int64(limit):
		reply.Infos = reply.Infos[offset:]
	case reply.Total-int64(offset) > int64(limit):
		reply.Infos = reply.Infos[offset : offset+limit]
	}
	return nil
}
