// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package access_log

import (
	"context"
	"gd_statistics/apis"
)

func providerServiceGetDataInfo(ctx context.Context, req *apis.BiGetProviderServiceDataInfoReq, start, end int64, isToday bool) (*apis.ProviderServiceDataInfo, error) {
	mreq := apis.BiGetProviderReportReq{}
	mreply := apis.BiGetProviderReportReply{}
	if req.ProviderApiId > 0 {
		mreq.ProviderApiId = []int64{req.ProviderApiId}
	}
	mreq.IsAll = true
	mreq.TodayEndTimestamp = req.TodayEndTimestamp
	mreq.Date = getDatesFromTimestamp(start, end)
	var err error
	if isToday {
		mreply.Infos, err = getTodayProviderReportFromHour(&mreq, start, end)
		if err != nil {
			return nil, err
		}
	} else {
		err = BiGetProviderReport(ctx, &mreq, &mreply)
		if err != nil {
			return nil, err
		}
	}
	ret := &apis.ProviderServiceDataInfo{}
	for _, v := range mreply.Infos {
		ret.Valid += v.Total
		ret.Charge += v.Charge
		ret.Fail += v.Fail
	}
	return ret, nil
}

func BiGetProviderServiceDataInfo(ctx context.Context, req *apis.BiGetProviderServiceDataInfoReq, reply *apis.BiGetProviderServiceDataInfoReply) (err error) {
	start, end := getTodayTimestamp()
	todayStart := start
	info, err := providerServiceGetDataInfo(ctx, req, start, end, true)
	if err != nil {
		return err
	}
	reply.Info.Today = *info

	start, end = getYesterdayTimestamp()
	info, err = providerServiceGetDataInfo(ctx, req, start, end, false)
	if err != nil {
		return err
	}
	reply.Info.Yesterday = *info

	start, end = getWeekTimestamp()
	end = todayStart
	if start == end {
		info = &apis.ProviderServiceDataInfo{}
	} else if info, err = providerServiceGetDataInfo(ctx, req, start, end, false); err != nil {
		return err
	}
	info.Charge += reply.Info.Today.Charge
	info.Valid += reply.Info.Today.Valid
	info.Fail += reply.Info.Today.Fail
	reply.Info.Week = *info

	start, end = getMonthTimestamp()
	end = todayStart
	if start == end {
		info = &apis.ProviderServiceDataInfo{}
	} else if info, err = providerServiceGetDataInfo(ctx, req, start, end, false); err != nil {
		return err
	}
	info.Charge += reply.Info.Today.Charge
	info.Valid += reply.Info.Today.Valid
	info.Fail += reply.Info.Today.Fail
	reply.Info.Month = *info
	return nil
}
