// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"fmt"
	"sort"
	"strings"
	"time"

	"github.com/astaxie/beego/orm"
)

type TGdExceptionMinute struct {
	ApiCodeTotal int64 `json:"api_code_total"`
	ApiValid     int   `json:"api_valid"`
	ApiCharge    int   `json:"api_charge"`
	ApiFailed    int   `json:"api_failed"`
	Codes        string
}

type ProviderExceptionMinute struct {
	Count           int64
	ProviderApiName string
}

func getOffSetToday(startTimestamp int64) int {
	nowTimestamp := time.Now().Unix()
	if nowTimestamp <= startTimestamp {
		return 0
	}
	if nowTimestamp-startTimestamp <= 24*3600 {
		return 1
	}
	if nowTimestamp-startTimestamp <= 6*24*3600 {
		return 6
	}
	return 7

}

func getTabName(startTimestamp int64) (string, string) {
	day := getOffSetToday(startTimestamp)
	if day == 0 {
		return "", ""
	}
	if day == 1 {
		return "t_gd_access_log_day", "t_gd_thirdpart_log_day"
	}
	if day == 6 {
		return "t_gd_access_log", "t_gd_thirdpart_access_log"
	}
	return "t_gd_access_log_history", "t_gd_thirdpart_access_log_history"
}

func BiGetDistributionNew(req *apis.BiGetDistributionReq, reply *apis.BiGetDistributionReply) error {
	if req.StartTimestamp == 0 || req.EndTimestamp == 0 {
		return errors.ArgsError
	}
	mreq := apis.LogQueryUserAccessCountExportReq{}
	mreq.MerchantId = req.MerchantId
	mreq.StartTimestamp = req.StartTimestamp
	mreq.EndTimestamp = req.EndTimestamp
	mreq.TabName = "t_gd_access_log_day"
	mreply := apis.LogQueryUserAccessCountExportReply{}
	if len(req.ApiId) != 0 {
		mreq.ApiIdList = append(mreq.ApiIdList, req.ApiId...)
	}
	err := LogQueryUserAccessCountExport(context.Background(), &mreq, &mreply)
	for _, v := range mreply.LogQueryUserAcessCount {
		reply.Info.ApiValid += int(v.Valid)
		reply.Info.ApiSuccess += int(v.Success)
		reply.Info.ApiFail += int(v.Total - v.Success)
		reply.Info.ApiCharge += int(v.Charge)
	}

	vreq := apis.LogQueryInterfaceCountReq{}
	vreq.ApiIds = req.ApiId
	vreq.MerchantId = req.MerchantId
	vreq.EndTimestamp = req.EndTimestamp
	vreq.StartTimestamp = req.StartTimestamp
	vreq.TabName = "t_gd_access_log_day"
	vreply := apis.LogQueryInterfaceAnalyzeErrorReply{}
	err = LogQueryInterfaceAnalyzeError(context.Background(), &vreq, &vreply)
	if err != nil {
		return err
	}
	total := 0
	for _, v := range vreply.List {
		item := apis.CodeCount{}
		item.Count = v.Count
		item.Code = fmt.Sprintf("%v", v.Code)
		item.Msg = v.Msg
		if item.Msg == "" && item.Code == "0" {
			item.Msg = "成功"
		}
		reply.Info.ApiCodeList = append(reply.Info.ApiCodeList, item)
		total += v.Count
	}
	reply.Info.ApiCodeTotal = int64(total)

	return nil
}

func getMaxHour(o orm.Ormer, tabs []string) (int64, error) {

	//sql := fmt.Sprintf("select min(a.hour) as hour from (subsql) a")
	subsql := ""
	max := int64(0)
	for _, v := range tabs {
		if subsql == "" {
			subsql = fmt.Sprintf("select max(hour) as hour from %s", v)
		} else {
			subsql = fmt.Sprintf("%s union (select max(hour) as hour from %s) ", subsql, v)
		}
	}
	if subsql == "" {
		return max, nil
	}
	sql := fmt.Sprintf("select min(a.hour) as hour from (%s) a", subsql)
	err := o.Raw(sql).QueryRow(&max)
	if err != nil && err != orm.ErrNoRows {
		return 0, errors.DataBaseError
	}
	return max, nil
}

func parseHourTimestamp(o orm.Ormer, start, end int64, tabs []string) (hourStart, hourEnd, otherStart, otherEnd int64, err error) {
	maxHour, err := getMaxHour(o, tabs)
	if err != nil {
		return 0, 0, 0, 0, err
	}

	// 小时归档表有该时段所有数据
	if maxHour+3600 >= end {
		return start, end, 0, 0, nil
	}

	// 小时归档表无该时段任何数据
	if maxHour < start {
		return 0, 0, start, end, nil
	}

	// 小时归档表有该时段部分数据
	return start, maxHour + 3600, maxHour + 3600, end, nil
}

func getApiDistributionNormal(o orm.Ormer, req *apis.BiGetDistributionReq, start, end int64) (*apis.BiDistribution, error) {
	reply := &apis.BiGetDistributionReply{}
	if start == 0 || end == 0 {
		return &reply.Info, nil
	}
	req.StartTimestamp = start
	req.EndTimestamp = end

	if true {
		if err := BiGetDistributionNew(req, reply); err != nil {
			return nil, err
		}
		return &reply.Info, nil
	}

	tabName, providerTabName := getTabName(req.StartTimestamp)
	if tabName == "" || providerTabName == "" {
		return &reply.Info, nil
	}

	mreq := apis.LogQueryUserAccessCountExportReq{}
	mreq.MerchantId = req.MerchantId
	mreq.StartTimestamp = req.StartTimestamp
	mreq.EndTimestamp = req.EndTimestamp
	mreply := apis.LogQueryUserAccessCountExportReply{}
	mreq.TabName = tabName
	if len(req.ApiId) != 0 {
		mreq.ApiIdList = append(mreq.ApiIdList, req.ApiId...)
	}
	err := LogQueryUserAccessCountExport(context.Background(), &mreq, &mreply)
	for _, v := range mreply.LogQueryUserAcessCount {
		reply.Info.ApiValid += int(v.Valid)
		reply.Info.ApiFail += int(v.Total - v.Success)
		reply.Info.ApiCharge += int(v.Charge)
	}

	vreq := apis.LogQueryInterfaceCountReq{}
	vreq.ApiIds = req.ApiId
	vreq.MerchantId = req.MerchantId
	vreq.EndTimestamp = req.EndTimestamp
	vreq.StartTimestamp = req.StartTimestamp
	vreq.TabName = tabName
	vreply := apis.LogQueryInterfaceAnalyzeErrorReply{}
	err = LogQueryInterfaceAnalyzeError(context.Background(), &vreq, &vreply)
	if err != nil {
		return nil, err
	}
	total := 0
	for _, v := range vreply.List {
		item := apis.CodeCount{}
		item.Count = v.Count
		item.Code = fmt.Sprintf("%v", v.Code)
		item.Msg = v.Msg
		if item.Msg == "" && item.Code == "0" {
			item.Msg = "成功"
		}
		reply.Info.ApiCodeList = append(reply.Info.ApiCodeList, item)
		total += v.Count
	}
	reply.Info.ApiCodeTotal = int64(total)

	return &reply.Info, nil
}

func getApiDistributionByHour(o orm.Ormer, req *apis.BiGetDistributionReq, start, end int64) (*apis.BiDistribution, error) {
	var charge, valid, fail int
	ret := &apis.BiDistribution{}
	if start == 0 || end == 0 {
		return ret, nil
	}
	sql := fmt.Sprintf("select sum(charge) as charge, sum(valid) as valid, sum(total-success) as fail from t_gd_report_hour where hour >= %d and hour < %d", start, end)
	if len(req.ApiId) > 0 {
		sql = fmt.Sprintf("%s and api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ApiId), "[]"), " ", ",", -1))
	}
	if req.MerchantId > 0 {
		sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
	}
	err := o.Raw(sql).QueryRow(&charge, &valid, &fail)
	if err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}

	ret.ApiCharge = charge
	ret.ApiValid = valid
	ret.ApiFail = fail

	sql = fmt.Sprintf("select code, msg, sum(count) as count from t_gd_api_code_hour where hour >= %d and hour < %d", start, end)
	if len(req.ApiId) > 0 {
		sql = fmt.Sprintf("%s and api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ApiId), "[]"), " ", ",", -1))
	}
	if req.MerchantId > 0 {
		sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
	}
	sql = fmt.Sprintf("%s group by code, msg", sql)
	_, err = o.Raw(sql).QueryRows(&ret.ApiCodeList)
	if err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}
	return ret, nil
}

func sortApiCode(array []apis.CodeCount) []apis.CodeCount {
	length := len(array)
	for i := 0; i < length; i++ {
		for j := i + 1; j < length; j++ {
			if array[i].Count < array[j].Count {
				tmp := array[i]
				array[i] = array[j]
				array[j] = tmp
			} else if array[i].Count == array[j].Count {
				nameArray := []string{
					array[i].Code + array[i].Msg,
					array[j].Code + array[j].Msg,
				}
				sort.Strings(nameArray)
				if (array[i].Code + array[i].Msg) != nameArray[0] {
					tmp := array[i]
					array[i] = array[j]
					array[j] = tmp
				}
			}
		}
	}
	return array
}

func BiGetDistribution(ctx context.Context, req *apis.BiGetDistributionReq, reply *apis.BiGetDistributionReply) error {
	if req.TodayEndTimestamp > 0 && req.EndTimestamp > req.TodayEndTimestamp {
		req.EndTimestamp = req.TodayEndTimestamp
	}
	o := orm.NewOrm()
	tabs := []string{
		"t_gd_report_hour",
		"t_gd_api_code_hour",
	}
	hourStart, hourEnd, otherStart, otherEnd, err := parseHourTimestamp(o, req.StartTimestamp, req.EndTimestamp, tabs)
	if err != nil {
		return err
	}
	hourInfos, err := getApiDistributionByHour(o, req, hourStart, hourEnd)
	if err != nil {
		return err
	}
	otherInfos, err := getApiDistributionNormal(o, req, otherStart, otherEnd)
	if err != nil {
		return err
	}
	reply.Info.ApiFail = (hourInfos.ApiFail + otherInfos.ApiFail)
	reply.Info.ApiValid = (hourInfos.ApiValid + otherInfos.ApiValid)
	reply.Info.ApiCharge = (hourInfos.ApiCharge + otherInfos.ApiCharge)

	m := map[string]*apis.CodeCount{}
	total := 0
	for i, v := range hourInfos.ApiCodeList {
		total += v.Count
		key := fmt.Sprintf("%s-%s", v.Code, v.Msg)
		if _, ok := m[key]; ok == false {
			m[key] = &hourInfos.ApiCodeList[i]
		}
	}
	for i, v := range otherInfos.ApiCodeList {
		total += v.Count
		key := fmt.Sprintf("%s-%s", v.Code, v.Msg)
		if _, ok := m[key]; ok == false {
			m[key] = &otherInfos.ApiCodeList[i]
			continue
		}
		m[key].Count += v.Count
	}
	reply.Info.ApiCodeTotal = int64(total)
	reply.Info.ApiCodeList = make([]apis.CodeCount, len(m))
	i := 0
	for _, v := range m {
		reply.Info.ApiCodeList[i] = *v
		i++
	}
	reply.Info.ApiCodeList = sortApiCode(reply.Info.ApiCodeList)
	return nil
}
