// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.
package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"fmt"
	"strings"

	"github.com/astaxie/beego/orm"
)

func BiGetProviderDistributionNew(req *apis.BiGetProviderDistributionReq, reply *apis.BiGetProviderDistributionReply) error {
	if req.StartTimestamp == 0 || req.EndTimestamp == 0 {
		return errors.ArgsError
	}
	mreq := apis.LogQueryProviderCountExportReq{}
	mreq.StartTimestamp = req.StartTimestamp
	mreq.EndTimestamp = req.EndTimestamp
	mreq.TabName = "t_gd_thirdpart_log_day"
	mreply := apis.LogQueryProviderCountExportReply{}
	if len(req.ProviderApiId) != 0 {
		mreq.ProviderApiIdList = append(mreq.ProviderApiIdList, req.ProviderApiId...)
	}
	err := LogQueryProviderCountExport(context.Background(), &mreq, &mreply)
	for _, v := range mreply.LogQueryProviderCount {
		reply.Info.ProviderCharge += int(v.Charge)
		reply.Info.ProviderFail += int(v.Failed)
	}

	vreq := apis.ThirdPartyInterfaceErrorAnalyzeReq{}
	vreq.ApiIds = req.ProviderApiId
	vreq.EndTimestamp = req.EndTimestamp
	vreq.StartTimestamp = req.StartTimestamp
	vreply := apis.ThirdPartyInterfaceErrorAnalyzeReply{}
	err = ThirdPartyInterfaceErrorAnalyze(context.Background(), &vreq, &vreply)
	if err != nil {
		return err
	}
	total := 0
	for _, v := range vreply.List {
		item := apis.CodeCount{}
		item.Count = v.Count
		item.Code = fmt.Sprintf("%v", v.Code)
		item.Msg = v.Msg
		reply.Info.ProviderCodeList = append(reply.Info.ProviderCodeList, item)
		total += v.Count
	}
	reply.Info.ProviderCodeTotal = total

	return nil
}

func getProviderDistributionByHour(o orm.Ormer, req *apis.BiGetProviderDistributionReq, start, end int64) (*apis.BiProviderDistribution, error) {
	var charge, fail int
	ret := &apis.BiProviderDistribution{}
	if start == 0 || end == 0 {
		return ret, nil
	}
	sql := fmt.Sprintf("select sum(charge) as charge, sum(failed) as fail from t_gd_provider_report_hour where hour >= %d and hour < %d", start, end)
	if len(req.ProviderApiIdNew) > 0 {
		sql = fmt.Sprintf("%s and provider_api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ProviderApiIdNew), "[]"), " ", ",", -1))
	}
	err := o.Raw(sql).QueryRow(&charge, &fail)
	if err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}

	ret.ProviderCharge = charge
	ret.ProviderFail = fail

	sql = fmt.Sprintf("select code, msg, sum(count) as count from t_gd_provider_code_hour where hour >= %d and hour < %d", start, end)
	if len(req.ProviderApiId) > 0 {
		sql = fmt.Sprintf("%s and provider_api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ProviderApiId), "[]"), " ", ",", -1))
	}
	sql = fmt.Sprintf("%s group by code, msg", sql)
	_, err = o.Raw(sql).QueryRows(&ret.ProviderCodeList)
	if err != nil && err != orm.ErrNoRows {
		return nil, errors.DataBaseError
	}
	return ret, nil
}

func getProviderDistributionNormal(o orm.Ormer, req *apis.BiGetProviderDistributionReq, start, end int64) (*apis.BiProviderDistribution, error) {
	reply := &apis.BiGetProviderDistributionReply{}
	if start == 0 || end == 0 {
		return &reply.Info, nil
	}
	req.StartTimestamp = start
	req.EndTimestamp = end

	if true {
		err := BiGetProviderDistributionNew(req, reply)
		if err != nil {
			return nil, err
		}
		return &reply.Info, nil
	}
	_, tabName := getTabName(req.StartTimestamp)
	if tabName == "" {
		return &reply.Info, nil
	}
	mreq := apis.LogQueryProviderCountExportReq{}
	mreq.StartTimestamp = req.StartTimestamp
	mreq.EndTimestamp = req.EndTimestamp
	mreq.TabName = tabName
	mreply := apis.LogQueryProviderCountExportReply{}
	if len(req.ProviderApiId) != 0 {
		mreq.ProviderApiIdList = append(mreq.ProviderApiIdList, req.ProviderApiId...)
	}
	err := LogQueryProviderCountExport(context.Background(), &mreq, &mreply)
	for _, v := range mreply.LogQueryProviderCount {
		reply.Info.ProviderCharge += int(v.Charge)
		reply.Info.ProviderFail += int(v.Failed)
	}

	vreq := apis.ThirdPartyInterfaceErrorAnalyzeReq{}
	vreq.ApiIds = req.ProviderApiId
	vreq.EndTimestamp = req.EndTimestamp
	vreq.StartTimestamp = req.StartTimestamp
	vreq.TabName = tabName
	vreply := apis.ThirdPartyInterfaceErrorAnalyzeReply{}
	err = ThirdPartyInterfaceErrorAnalyze(context.Background(), &vreq, &vreply)
	if err != nil {
		return nil, err
	}
	total := 0
	for _, v := range vreply.List {
		item := apis.CodeCount{}
		item.Count = v.Count
		item.Code = fmt.Sprintf("%v", v.Code)
		item.Msg = v.Msg
		reply.Info.ProviderCodeList = append(reply.Info.ProviderCodeList, item)
		total += v.Count
	}
	reply.Info.ProviderCodeTotal = total
	return &reply.Info, nil
}

func BiGetProviderDistribution(ctx context.Context, req *apis.BiGetProviderDistributionReq, reply *apis.BiGetProviderDistributionReply) error {
	if req.TodayEndTimestamp > 0 && req.EndTimestamp > req.TodayEndTimestamp {
		req.EndTimestamp = req.TodayEndTimestamp
	}
	o := orm.NewOrm()
	req.ProviderApiIdNew = req.ProviderApiId
	tabs := []string{
		"t_gd_provider_report_hour",
		"t_gd_provider_code_hour",
	}
	hourStart, hourEnd, otherStart, otherEnd, err := parseHourTimestamp(o, req.StartTimestamp, req.EndTimestamp, tabs)
	if err != nil {
		return err
	}
	hourInfos, err := getProviderDistributionByHour(o, req, hourStart, hourEnd)
	if err != nil {
		return err
	}
	otherInfos, err := getProviderDistributionNormal(o, req, otherStart, otherEnd)
	if err != nil {
		return err
	}

	reply.Info.ProviderFail = (hourInfos.ProviderFail + otherInfos.ProviderFail)
	reply.Info.ProviderCharge = (hourInfos.ProviderCharge + otherInfos.ProviderCharge)

	m := map[string]*apis.CodeCount{}
	total := 0
	for i, v := range hourInfos.ProviderCodeList {
		total += v.Count
		key := fmt.Sprintf("%s-%s", v.Code, v.Msg)
		if _, ok := m[key]; ok == false {
			m[key] = &hourInfos.ProviderCodeList[i]
		}
	}
	for i, v := range otherInfos.ProviderCodeList {
		total += v.Count
		key := fmt.Sprintf("%s-%s", v.Code, v.Msg)
		if _, ok := m[key]; ok == false {
			m[key] = &otherInfos.ProviderCodeList[i]
			continue
		}
		m[key].Count += v.Count
	}
	reply.Info.ProviderCodeTotal = total
	reply.Info.ProviderCodeList = make([]apis.CodeCount, len(m))
	i := 0
	for _, v := range m {
		reply.Info.ProviderCodeList[i] = *v
		i++
	}
	reply.Info.ProviderCodeList = sortApiCode(reply.Info.ProviderCodeList)
	return nil
}
