package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"fmt"
	"github.com/astaxie/beego/orm"
	"strconv"
	"time"
)

func getAccesslogCountWhere(req *apis.LogQueryAccessLogCountReq) (w string, s []interface{}) {
	if req.EndTimestamp != 0 && req.StartTimestamp != 0 {
		w += " and " + "timestamp >=" + " ? and timestamp < ?"
		s = append(s, req.StartTimestamp, req.EndTimestamp)
	} else {
		nowTime := time.Now()
		zeroTime := time.Date(nowTime.Year(), nowTime.Month(), nowTime.Day(), 0, 0, 0, 0, nowTime.Location())
		startTimeStamp := zeroTime.Unix()
		endTimeStamp := nowTime.Unix()
		w += " and " + "timestamp >=" + " ? and timestamp < ?"
		s = append(s, startTimeStamp, endTimeStamp)
	}
	if req.ApiId != 0 {
		w += " and " + "api_id" + " = ?"
		s = append(s, req.ApiId)
	}
	if req.MerchantId != 0 {
		w += " and " + "merchant_id" + " = ?"
		s = append(s, req.MerchantId)
	}

	if len(s) > 0 {
		//有条件才截取最前面的and
		w = string([]byte(w)[4:])
		w = " where" + w
		return w, s
	} else {
		return "", nil
	}

}

func LogQueryAccessLogCountNew(ctx context.Context, req *apis.LogQueryAccessLogCountReq, reply *apis.LogQueryAccessLogCountReply) error {
	if req.StartTimestamp == 0 || req.EndTimestamp == 0 {
		nowTime := time.Now()
		zeroTime := time.Date(nowTime.Year(), nowTime.Month(), nowTime.Day(), 0, 0, 0, 0, nowTime.Location())
		req.StartTimestamp = zeroTime.Unix()
		req.EndTimestamp = nowTime.Unix()
	}

	exportTimes := getExportTimes(req.StartTimestamp, req.EndTimestamp)
	for _, v := range exportTimes {
		req.StartTimestamp = v.start
		req.EndTimestamp = v.end
		tabName := getMonthTab("t_gd_access_log_month", req.StartTimestamp)
		subReply := &apis.LogQueryAccessLogCountReply{}
		err := logQueryAccessLogCount(req, subReply, tabName)
		if err != nil {
			return err
		}
		reply.QueryAccessLogCounts = append(reply.QueryAccessLogCounts, subReply.QueryAccessLogCounts...)
	}

	if req.PageSize == 0 {
		req.PageSize = PAGESIZE
	}
	if req.PageNumber == 0 {
		req.PageNumber = 1
	}
	reply.Total = int64(len(reply.QueryAccessLogCounts))
	offset := req.PageSize * (req.PageNumber - 1)

	switch {
	case reply.Total-offset <= 0:
		reply.QueryAccessLogCounts = []apis.QueryAccessLogCount{}
	case reply.Total-offset <= req.PageSize:
		reply.QueryAccessLogCounts = reply.QueryAccessLogCounts[offset:]
	case reply.Total-offset > req.PageSize:
		reply.QueryAccessLogCounts = reply.QueryAccessLogCounts[offset : offset+req.PageSize]
	}

	return nil
}

func logQueryAccessLogCount(req *apis.LogQueryAccessLogCountReq, reply *apis.LogQueryAccessLogCountReply, tabName string) error {
	where, val := getAccesslogCountWhere(req)
	sqlStr := "select create_time as date,merchant_name,api_name,count(state) as total ,sum(state) as success ,sum(is_reuse and code=0) as reuse,avg(elapsed) as elapsed  from " + tabName + " %s group by create_time,merchant_id,api_id order by create_time desc"

	sqlStr = fmt.Sprintf(sqlStr, where)
	_, err := orm.NewOrm().Raw(sqlStr, val).QueryRows(&reply.QueryAccessLogCounts)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}
	for index, _ := range reply.QueryAccessLogCounts {
		reply.QueryAccessLogCounts[index].Failed = reply.QueryAccessLogCounts[index].Total - reply.QueryAccessLogCounts[index].Success
		reply.QueryAccessLogCounts[index].SuccessRate = strconv.FormatFloat(float64(100*reply.QueryAccessLogCounts[index].Success)/float64(reply.QueryAccessLogCounts[index].Total), 'f', 2, 64) + "%"
		reply.QueryAccessLogCounts[index].ReuseRate = strconv.FormatFloat(float64(100*reply.QueryAccessLogCounts[index].Reuse)/float64(reply.QueryAccessLogCounts[index].Total), 'f', 2, 64) + "%"
	}
	return nil
}

func LogQueryAccessLogCount(ctx context.Context, req *apis.LogQueryAccessLogCountReq, reply *apis.LogQueryAccessLogCountReply) error {
	if true {
		return LogQueryAccessLogCountNew(ctx, req, reply)
	}

	if req.PageSize == 0 {
		req.PageSize = PAGESIZE
	}

	if req.PageNumber == 0 {
		req.PageNumber = 1
	}

	limit := strconv.FormatInt((req.PageNumber-1)*req.PageSize, 10) + "," + strconv.FormatInt(req.PageSize, 10)

	where, val := getAccesslogCountWhere(req)
	sqlStr := "select create_time as date,merchant_name,api_name,count(state) as total ,sum(state) as success ,sum(is_reuse and code=0) as reuse,avg(elapsed) as elapsed  from t_gd_access_log %s group by create_time,merchant_id,api_id order by create_time desc limit %s"

	sqlStr = fmt.Sprintf(sqlStr, where, limit)

	var ids []int

	count, err := orm.NewOrm().Raw("select id from t_gd_access_log "+where+" group by create_time,merchant_id,api_id", val).QueryRows(&ids)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}

	reply.Total = count
	_, err = orm.NewOrm().Raw(sqlStr, val).QueryRows(&reply.QueryAccessLogCounts)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}

	for index, _ := range reply.QueryAccessLogCounts {
		reply.QueryAccessLogCounts[index].Failed = reply.QueryAccessLogCounts[index].Total - reply.QueryAccessLogCounts[index].Success
		reply.QueryAccessLogCounts[index].SuccessRate = strconv.FormatFloat(float64(100*reply.QueryAccessLogCounts[index].Success)/float64(reply.QueryAccessLogCounts[index].Total), 'f', 2, 64) + "%"
		reply.QueryAccessLogCounts[index].ReuseRate = strconv.FormatFloat(float64(100*reply.QueryAccessLogCounts[index].Reuse)/float64(reply.QueryAccessLogCounts[index].Total), 'f', 2, 64) + "%"
	}

	reply.PageNumber = req.PageNumber
	reply.PageSize = req.PageSize
	return nil
}
