package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"gd_statistics/rpc_apis"
	"gd_statistics/rpc_apis/gd_management"
	"fmt"
	"strconv"
	"strings"
	"time"

	"github.com/astaxie/beego/orm"
)

func getProviderCountWhereExport(req *apis.LogQueryProviderCountExportReq) (w string, s []interface{}) {
	if req.EndTimestamp != 0 && req.StartTimestamp != 0 {
		w += " and " + "timestamp >=" + " ? and timestamp <?"
		s = append(s, req.StartTimestamp, req.EndTimestamp)
	} else {
		nowTime := time.Now()
		zeroTime := time.Date(nowTime.Year(), nowTime.Month(), nowTime.Day(), 0, 0, 0, 0, nowTime.Location())
		startTimeStamp := zeroTime.Unix()
		endTimeStamp := nowTime.Unix()

		w += " and " + "timestamp >=" + " ? and timestamp <?"
		s = append(s, startTimeStamp, endTimeStamp)
	}
	if len(req.ProviderApiIdList) != 0 {
		w += " and provider_api_id in (" + strings.Replace(strings.Trim(fmt.Sprint(req.ProviderApiIdList), "[]"), " ", ",", -1) + ")"
	}
	if req.ProviderId != 0 {
		//strings.Replace(strings.Trim(fmt.Sprint(array_or_slice), "[]"), " ", ",", -1)
		w += " and " + "provider_id" + " = ?"
		s = append(s, req.ProviderId)
	}
	if req.MerchantId != 0 {
		//strings.Replace(strings.Trim(fmt.Sprint(array_or_slice), "[]"), " ", ",", -1)
		w += " and " + "merchant_id" + " = ?"
		s = append(s, req.MerchantId)
	}
	if req.ApiId != 0 {
		w += " and " + "api_id" + " = ?"
		s = append(s, req.ApiId)
	}
	if len(req.ApiIds) > 0 {
		w = fmt.Sprintf("%s and api_id in(%s)", w, strings.Replace(strings.Trim(fmt.Sprint(req.ApiIds), "[]"), " ", ",", -1))
	}

	if len(s) > 0 {
		//有条件才截取最前面的and
		w = string([]byte(w)[4:])
		w = " where" + w
		return w, s
	} else {
		return "", nil
	}

}

func getProviderExportSql(req *apis.LogQueryProviderCountExportReq, where string) string {

	sqlStr := "select create_time as date, provider_api_id, count(state) as total, " +
		"sum(state=1) as success, sum(code<>1100 and state=1) as query, " +
		"sum(state<>1) as failed," +
		"sum(code=1100) as query_no_record, " +
		"avg(elapsed) as avg_elapsed, " +
		"sum(elapsed) as sum_elapsed" +
		"%s" +
		" from " + req.TabName + " as a %s group by create_time,provider_api_id order by timestamp"

	concat := ""
	subSql := ""
	timeList := strings.Split(req.TimeList, ",")
	for _, v := range timeList {
		if v == "" {
			break
		}
		if subSql == "" {
			subSql = fmt.Sprintf("sum(elapsed > %s)", v)
			continue
		}
		subSql = fmt.Sprintf("%s,sum(elapsed > %s)", subSql, v)
	}
	if subSql != "" {
		concat = fmt.Sprintf(",concat_ws(',', %s) as big_elapseds", subSql)
	}

	sqlStr = fmt.Sprintf(sqlStr, concat, where)
	return sqlStr
}

type ProviderCountInfo struct {
	ProviderApiId   int64  `json:"provider_api_id"`
	CountType       int    `json:"count_type"`
	CountKey        string `json:"count_key"`
	CountValue      string `json:"count_value"`
	RawCode         string `json:"raw_code"`
	ProviderName    string `json:"provider_name"`
	ProviderApiName string `json:"provider_api_name"`
}

func getProviderCountInfo() (map[int64]ProviderCountInfo, error) {
	if globalProviderCountInfo != nil {
		return globalProviderCountInfo, nil
	}
	req := gd_management.ManagementGetProviderCountInfoReq{}
	reply, err := rpc_apis.Management.ManagementGetProviderCountInfo(context.Background(), &req)
	ret := map[int64]ProviderCountInfo{}
	for _, v := range reply.Infos {
		item := ProviderCountInfo{}
		item.ProviderApiId = v.ProviderApiId
		item.CountType = v.CountType
		item.CountKey = v.CountKey
		item.CountValue = v.CountValue
		item.ProviderName = v.ProviderName
		item.ProviderApiName = v.ProviderApiName
		delimer := ","
		if strings.Contains(v.CountValue, ";") {
			delimer = ";"
		}
		array := strings.Split(v.CountValue, delimer)
		for _, code := range array {
			if item.RawCode == "" {
				item.RawCode = fmt.Sprintf("'%s'", code)
				continue
			}
			item.RawCode = fmt.Sprintf("%s,'%s'", item.RawCode, code)
		}
		ret[v.ProviderApiId] = item
	}
	return ret, err
}

func getProviderIdTotal(id int64, data []apis.ProviderApiIdTotal) (int64, string) {
	for _, v := range data {
		if v.ProviderApiId == id {
			return v.Total, v.BigElapseds
		}
	}
	return 0, ""
}
func computeProviderSpecialCharge(o orm.Ormer, req *apis.LogQueryProviderCountExportReq,
	info *apis.LogQueryProviderCountExport, countInfos map[int64]ProviderCountInfo) error {

	if req.NoCharge {
		return nil
	}

	concat := ""
	subSql := ""
	timeList := strings.Split(req.TimeList, ",")
	for _, v := range timeList {
		if v == "" {
			break
		}
		if subSql == "" {
			subSql = fmt.Sprintf("sum(elapsed > %s)", v)
			continue
		}
		subSql = fmt.Sprintf("%s,sum(elapsed > %s)", subSql, v)
	}
	if subSql != "" {
		concat = fmt.Sprintf(",concat_ws(',', %s) as big_elapseds", subSql)
	}
	info.ChargeBigElapsedList = make([]int, len(timeList))

	count := int64(0)
	for _, id := range info.ProviderApiIds {
		chargeBigElapseds := ""
		if countInfo, ok := countInfos[id]; ok {
			if countInfo.CountType == 0 {
				count, chargeBigElapseds = getProviderIdTotal(id, info.ProviderApiIdTotals)
			} else if countInfo.CountValue == "" {
				count = 0
				for i := 0; i < len(timeList); i++ {
					if chargeBigElapseds == "" {
						chargeBigElapseds = "0"
					} else {
						chargeBigElapseds = fmt.Sprintf("%s,0", chargeBigElapseds)
					}
				}
			} else {
				count = int64(0)
				start, end := getStartEndTimestamp(req.StartTimestamp, req.EndTimestamp, info.Date)
				sql := fmt.Sprintf("select count(1)"+concat+" from "+req.TabName+" where state=1 and raw_code in(%s) and provider_api_id=%d and timestamp >= %d and timestamp < %d", countInfo.RawCode, id, start, end)
				if req.MerchantId > 0 {
					sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
				}
				if len(req.ApiIds) > 0 {
					sql = fmt.Sprintf("%s and api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ApiIds), "[]"), " ", ",", -1))
				}
				if concat == "" {
					err := o.Raw(sql).QueryRow(&count)
					if err != nil {
						return errors.DataBaseError
					}
				} else {
					err := o.Raw(sql).QueryRow(&count, &chargeBigElapseds)
					if err != nil {
						return errors.DataBaseError
					}

				}
			}
		} else {
			count, chargeBigElapseds = getProviderIdTotal(id, info.ProviderApiIdTotals)
		}
		info.Charge += count

		if chargeBigElapseds != "" {
			array := strings.Split(chargeBigElapseds, ",")
			for i, _ := range array {
				vint, _ := strconv.Atoi(array[i])
				info.ChargeBigElapsedList[i] += vint
			}
		}

	}
	info.ChargeRate = strconv.FormatFloat(float64(100*count)/float64(info.Total), 'f', 2, 64) + "%"
	return nil
}

func computeProviderCharge(o orm.Ormer, req *apis.LogQueryProviderCountExportReq,
	info *apis.LogQueryProviderCountExport, countInfos map[int64]ProviderCountInfo) error {
	if len(info.ProviderApiIds) > 0 {
		return computeProviderSpecialCharge(o, req, info, countInfos)
	}
	if req.NoCharge {
		return nil
	}
	concat := ""
	subSql := ""
	timeList := strings.Split(req.TimeList, ",")
	for _, v := range timeList {
		if v == "" {
			break
		}
		if subSql == "" {
			subSql = fmt.Sprintf("sum(elapsed > %s)", v)
			continue
		}
		subSql = fmt.Sprintf("%s,sum(elapsed > %s)", subSql, v)
	}
	if subSql != "" {
		concat = fmt.Sprintf(",concat_ws(',', %s) as big_elapseds", subSql)
	}
	chargeBigElapseds := ""
	info.ChargeBigElapsedList = make([]int, len(timeList))
	if countInfo, ok := countInfos[info.ProviderApiId]; ok {
		if countInfo.CountType == 0 {
			info.Charge = info.Total
			info.ChargeRate = "100%"
			chargeBigElapseds = info.BigElapseds
		} else if countInfo.CountValue == "" {
			info.Charge = 0
			info.ChargeRate = "0%"
			for i := 0; i < len(timeList); i++ {
				if chargeBigElapseds == "" {
					chargeBigElapseds = "0"
				} else {
					chargeBigElapseds = fmt.Sprintf("%s,0", chargeBigElapseds)
				}
			}
		} else {
			start, end := getStartEndTimestamp(req.StartTimestamp, req.EndTimestamp, info.Date)
			count := int64(0)
			sql := fmt.Sprintf("select count(1) "+concat+" from "+req.TabName+" where raw_code in(%s) and provider_api_id=%d and timestamp >= %d and timestamp < %d", countInfo.RawCode, info.ProviderApiId, start, end)
			if req.MerchantId > 0 {
				sql = fmt.Sprintf("%s and merchant_id=%d", sql, req.MerchantId)
			}
			if len(req.ApiIds) > 0 {
				sql = fmt.Sprintf("%s and api_id in(%s)", sql, strings.Replace(strings.Trim(fmt.Sprint(req.ApiIds), "[]"), " ", ",", -1))
			}
			if concat == "" {
				err := o.Raw(sql).QueryRow(&count)
				if err != nil {
					return errors.DataBaseError
				}
			} else {
				err := o.Raw(sql).QueryRow(&count, &chargeBigElapseds)
				if err != nil {
					return errors.DataBaseError
				}
			}
			info.Charge = count
			info.ChargeRate = strconv.FormatFloat(float64(100*count)/float64(info.Total), 'f', 2, 64) + "%"
		}
	} else {
		info.Charge = info.Total
		info.ChargeRate = "100%"
		chargeBigElapseds = info.BigElapseds
	}

	if chargeBigElapseds != "" {
		array := strings.Split(chargeBigElapseds, ",")
		for i, _ := range array {
			vint, _ := strconv.Atoi(array[i])
			info.ChargeBigElapsedList[i] += vint
		}
	}
	return nil
}

func logQueryProviderCountExport(req *apis.LogQueryProviderCountExportReq, reply *apis.LogQueryProviderCountExportReply) error {
	o := orm.NewOrm()
	where, val := getProviderCountWhereExport(req)
	sqlStr := getProviderExportSql(req, where)

	_, err := o.Raw(sqlStr, val).QueryRows(&reply.LogQueryProviderCount)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}
	countInfos, _ := getProviderCountInfo()

	for i, v := range reply.LogQueryProviderCount {
		if info, ok := countInfos[v.ProviderApiId]; ok {
			reply.LogQueryProviderCount[i].ProviderName = info.ProviderName
			reply.LogQueryProviderCount[i].ProviderApiName = info.ProviderApiName
		}
	}

	for index, info := range reply.LogQueryProviderCount {
		if err := computeProviderCharge(o, req, &reply.LogQueryProviderCount[index], countInfos); err != nil {
			return err
		}
		if info.BigElapseds != "" {
			array := strings.Split(info.BigElapseds, ",")
			reply.LogQueryProviderCount[index].BigElapsedList = make([]int, len(array))
			for i, _ := range array {
				vint, _ := strconv.Atoi(array[i])
				reply.LogQueryProviderCount[index].BigElapsedList[i] = vint
			}
		}
		reply.LogQueryProviderCount[index].Failed = info.Total - info.Success

		if info.Total > 0 {
			reply.LogQueryProviderCount[index].FailedRate = strconv.FormatFloat(float64(100*info.Failed)/float64(info.Total), 'f', 2, 64) + "%"
			reply.LogQueryProviderCount[index].QueryNoRecordRate = strconv.FormatFloat(float64(100*info.QueryNoRecord)/float64(info.Total), 'f', 2, 64) + "%"
		} else {
			reply.LogQueryProviderCount[index].FailedRate = "0%"
			reply.LogQueryProviderCount[index].QueryNoRecordRate = "0%"
		}
	}

	return nil
}

func LogQueryProviderCountExportNew(req *apis.LogQueryProviderCountExportReq, reply *apis.LogQueryProviderCountExportReply) error {
	globalProviderExportMux.RLock()
	defer globalProviderExportMux.RUnlock()
	if req.EndTimestamp == 0 || req.StartTimestamp == 0 {
		now := time.Now()
		req.EndTimestamp = now.Unix()
		req.StartTimestamp = time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location()).Unix()
	}
	if req.TabName == "t_gd_thirdpart_log_day" {
		return logQueryProviderCountExport(req, reply)
	}

	exportTimes := getExportTimes(req.StartTimestamp, req.EndTimestamp)
	for _, v := range exportTimes {
		req.StartTimestamp = v.start
		req.EndTimestamp = v.end
		req.TabName = getMonthTab("t_gd_thirdpart_log_month", req.StartTimestamp)
		subReply := &apis.LogQueryProviderCountExportReply{}
		err := logQueryProviderCountExport(req, subReply)
		if err != nil {
			return err
		}
		reply.LogQueryProviderCount = append(reply.LogQueryProviderCount, subReply.LogQueryProviderCount...)
	}
	return nil

}

func LogQueryProviderCountExport(ctx context.Context, req *apis.LogQueryProviderCountExportReq, reply *apis.LogQueryProviderCountExportReply) error {
	return LogQueryProviderCountExportNew(req, reply)

}
