// Copyright 2019 utimes.cc. All rights reserved.
// Use of this source code is governed by utimes.cc.

package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"encoding/json"
	"fmt"
	"strings"
	"time"

	"github.com/astaxie/beego/orm"
)

func getThirdpartAccesslogWhere(req *apis.LogQueryThirdpartAccessLogReq, querySet orm.QuerySeter) orm.QuerySeter {
	cond := orm.NewCondition()
	if req.EndTimestamp != 0 && req.StartTimestamp != 0 {
		cond = cond.And("timestamp__gte", req.StartTimestamp).And("timestamp__lte", req.EndTimestamp)
	}
	if req.ProviderId != 0 {
		cond = cond.And("provider_id", req.ProviderId)
	}

	if req.State != 0 {
		realState := 1
		if req.State == 2 {
			realState = 0
		}
		cond = cond.And("state", realState)
	}
	if req.ProviderApiId != 0 {
		cond = cond.And("provider_api_id", req.ProviderApiId)
	}

	if req.RawCode != "" {
		cond = cond.And("raw_code", req.RawCode)
	}

	if req.Search != "" {
		cond = cond.And("search", req.Search)
	} else if req.RequestParam != "" {
		cond1 := orm.NewCondition()
		cond = cond.AndCond(cond1.And("request_params__contains", req.RequestParam).Or("response_params__contains", req.RequestParam))
	}

	if req.Max != 0 || req.Min != 0 {
		switch {
		case req.Min != 0 && req.Max != 0:
			cond = cond.And("elapsed__gte", float64(req.Min)).And("elapsed__lte", float64(req.Max))
		case req.Min != 0:
			cond = cond.And("elapsed__gte", float64(req.Min))
		case req.Max != 0:
			cond = cond.And("elapsed__lte", float64(req.Max))
		}
	}

	querySet = querySet.SetCond(cond)
	return querySet
}

func getThirdpartAccesslogWhereSql(req *apis.LogQueryThirdpartAccessLogReq, tabName string) string {
	sql := fmt.Sprintf("select * from %s where timestamp >= %d and timestamp <= %d", tabName, req.StartTimestamp, req.EndTimestamp)
	if req.ProviderApiId != 0 {
		sql = fmt.Sprintf("%s and provider_api_id=%d", sql, req.ProviderApiId)
	}
	if req.State != 0 {
		realState := 1
		if req.State == 2 {
			realState = 0
		}
		sql = fmt.Sprintf("%s and state=%d", sql, realState)
	}
	if req.ProviderId != 0 {
		sql = fmt.Sprintf("%s and provider_id=%d", sql, req.ProviderId)
	}

	if req.RawCode != "" {
		sql = fmt.Sprintf("%s and raw_code='%s'", sql, req.RawCode)
	}

	if req.Search != "" {
		sql = fmt.Sprintf("%s and search='%s'", sql, req.Search)
	} else if req.RequestParam != "" {
		sql = fmt.Sprintf("%s and (request_params like '%s' or response_params like '%s')", sql, "%"+req.RequestParam+"%", "%"+req.RequestParam+"%")
	}
	if req.Max != 0 || req.Min != 0 {
		switch {
		case req.Min != 0 && req.Max != 0:
			sql = fmt.Sprintf("%s and elapsed >= %f and elapsed <= %f", sql, req.Min, req.Max)
		case req.Min != 0:
			sql = fmt.Sprintf("%s and elapsed >= %f", sql, req.Min)
		case req.Max != 0:
			sql = fmt.Sprintf("%s and elapsed <= %f", sql, req.Max)
		}
	}
	return sql
}

func LogQueryThirdpartAccessLogNew(req *apis.LogQueryThirdpartAccessLogReq, reply *apis.LogQueryThirdpartAccessLogReply) error {
	if req.StartTimestamp <= 0 || req.EndTimestamp <= 0 {
		now := time.Now()
		req.StartTimestamp = time.Date(now.Year(), now.Month(), now.Day(), 0, 0, 0, 0, now.Location()).Unix()
		req.EndTimestamp = time.Date(now.Year(), now.Month(), now.Day(), 23, 59, 59, 0, now.Location()).Unix()
	}

	mTabs := getMonthTabs("t_gd_thirdpart_log_month", req.StartTimestamp, req.EndTimestamp)
	o := orm.NewOrm()
	if req.PageSize == 0 {
		req.PageSize = PAGESIZE
	}

	if req.PageNumber == 0 {
		req.PageNumber = 1
	}
	oldCount := (req.PageNumber - 1) * req.PageSize
	for i, v := range mTabs {
		mreq := *req
		mreq.StartTimestamp = v.StartTimestamp
		mreq.EndTimestamp = v.EndTimestamp
		querySet := o.QueryTable(v.TabName)
		sql := getThirdpartAccesslogWhereSql(&mreq, v.TabName)
		querySet = getThirdpartAccesslogWhere(&mreq, querySet)
		count, err := querySet.Count()
		if err != nil {
			return errors.DataBaseError
		}
		mTabs[i].Count = count
		reply.Total += count
		if reply.Total <= oldCount {
			continue
		}
		list := []apis.ThirdpartLog{}
		if int64(len(reply.ThirdpartLogs)) < req.PageSize {
			offset := oldCount - (reply.Total - count)
			_, err = o.Raw(sql+" order by timestamp desc limit ? offset ?", req.PageSize-int64(len(reply.ThirdpartLogs)), offset).QueryRows(&list)
			if err != nil && err != orm.ErrNoRows {
				return errors.DataBaseError
			}
			reply.ThirdpartLogs = append(reply.ThirdpartLogs, list...)
			oldCount += int64(len(list))
		}
	}
	reply.PageNumber = req.PageNumber
	reply.PageSize = req.PageSize

	if req.IsExport {
		return nil
	}
	for i, v := range reply.ThirdpartLogs {
		var timeLayout = "2006-01-02 15:04:05"
		reply.ThirdpartLogs[i].CreateTime = time.Unix(reply.ThirdpartLogs[i].Timestamp, 0).Format(timeLayout)
		if strings.Contains(v.ResponseParams, "{") {
			m := map[string]interface{}{}
			json.Unmarshal([]byte(v.ResponseParams), &m)
			bytes, err := json.Marshal(m)
			if err == nil {
				reply.ThirdpartLogs[i].ResponseParams = string(bytes)
			}

			array := []map[string]interface{}{}
			bytes = []byte(v.ResponseParams)
			json.Unmarshal(bytes, &array)
			bytes, _ = json.Marshal(array)
			if len(array) > 0 {
				reply.ThirdpartLogs[i].ResponseParams = string(bytes)
			}
			m = nil
			array = nil

		}
	}

	return nil
}

func LogQueryThirdpartAccessLog(ctx context.Context, req *apis.LogQueryThirdpartAccessLogReq, reply *apis.LogQueryThirdpartAccessLogReply) error {
	return LogQueryThirdpartAccessLogNew(req, reply)
}
