package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/errors"
	"fmt"
	"strconv"
	"time"

	"github.com/astaxie/beego/orm"
)

func getLogQueryThirdpartInterfaceCountWhere(req *apis.LogQueryThirdpartInterfaceCountReq) (w string, s []interface{}) {
	if req.EndTimestamp != 0 && req.StartTimestamp != 0 {
		w += " and " + "timestamp >=" + " ? and timestamp <?"
		s = append(s, req.StartTimestamp, req.EndTimestamp)
	} else {
		nowTime := time.Now()
		zeroTime := time.Date(nowTime.Year(), nowTime.Month(), nowTime.Day(), 0, 0, 0, 0, nowTime.Location())
		startTimeStamp := zeroTime.Unix()
		endTimeStamp := nowTime.Unix()
		w += " and " + "timestamp >=" + " ? and timestamp <?"
		s = append(s, startTimeStamp, endTimeStamp)
	}
	if req.ProviderApiId != 0 {
		w += " and " + "provider_api_id" + " = ?"
		s = append(s, req.ProviderApiId)
	}

	if len(s) > 0 {
		//有条件才截取最前面的and
		w = string([]byte(w)[4:])
		w = " where" + w
		return w, s
	} else {
		return "", nil
	}
}

func LogQueryThirdpartInterfaceCountNew(ctx context.Context, req *apis.LogQueryThirdpartInterfaceCountReq, reply *apis.LogQueryThirdpartInterfaceCountReply) error {
	if req.StartTimestamp == 0 || req.EndTimestamp == 0 {
		nowTime := time.Now()
		zeroTime := time.Date(nowTime.Year(), nowTime.Month(), nowTime.Day(), 0, 0, 0, 0, nowTime.Location())
		req.StartTimestamp = zeroTime.Unix()
		req.EndTimestamp = nowTime.Unix()
	}

	exportTimes := getExportTimes(req.StartTimestamp, req.EndTimestamp)
	m := map[int64]*apis.LogQueryThirdpartInterfaceCount{}
	for _, v := range exportTimes {
		req.StartTimestamp = v.start
		req.EndTimestamp = v.end
		tabName := getMonthTab("t_gd_thirdpart_log_month", req.StartTimestamp)
		subReply := &apis.LogQueryThirdpartInterfaceCountReply{}
		err := logQueryThirdpartInterfaceCount(req, subReply, tabName)
		if err != nil {
			return err
		}
		for i, v := range subReply.LogQueryThirdpartInterfaceCount {
			if _, ok := m[v.ProviderApiId]; ok == false {
				m[v.ProviderApiId] = &subReply.LogQueryThirdpartInterfaceCount[i]
				continue
			}
			m[v.ProviderApiId].Total += v.Total
			m[v.ProviderApiId].Success += v.Success
			m[v.ProviderApiId].Query += v.Query
			m[v.ProviderApiId].Fail += v.Fail
			m[v.ProviderApiId].UnQuery += v.UnQuery
		}
	}
	if req.PageSize == 0 {
		req.PageSize = PAGESIZE
	}
	if req.PageNumber == 0 {
		req.PageNumber = 1
	}
	reply.Total = int64(len(m))
	reply.LogQueryThirdpartInterfaceCount = make([]apis.LogQueryThirdpartInterfaceCount, len(m))
	i := 0
	for _, v := range m {
		reply.LogQueryThirdpartInterfaceCount[i].Total = v.Total
		reply.LogQueryThirdpartInterfaceCount[i].ProviderApiId = v.ProviderApiId
		reply.LogQueryThirdpartInterfaceCount[i].UnQuery = v.UnQuery
		reply.LogQueryThirdpartInterfaceCount[i].Fail = v.Fail
		reply.LogQueryThirdpartInterfaceCount[i].Query = v.Query
		reply.LogQueryThirdpartInterfaceCount[i].Success = v.Success
		reply.LogQueryThirdpartInterfaceCount[i].Date = v.Date
		reply.LogQueryThirdpartInterfaceCount[i].ProviderApiName = v.ProviderApiName
		reply.LogQueryThirdpartInterfaceCount[i].ProviderName = v.ProviderName
		if reply.LogQueryThirdpartInterfaceCount[i].Total != 0 {
			reply.LogQueryThirdpartInterfaceCount[i].FailRate = strconv.FormatFloat(float64(100*reply.LogQueryThirdpartInterfaceCount[i].Fail)/float64(reply.LogQueryThirdpartInterfaceCount[i].Total), 'f', 2, 64) + "%"
		} else {
			reply.LogQueryThirdpartInterfaceCount[i].FailRate = "0%"
		}
		if reply.LogQueryThirdpartInterfaceCount[i].Success != 0 {
			reply.LogQueryThirdpartInterfaceCount[i].QueryRate = strconv.FormatFloat(float64(100*reply.LogQueryThirdpartInterfaceCount[i].Query)/float64(reply.LogQueryThirdpartInterfaceCount[i].Success), 'f', 2, 64) + "%"
		} else {
			reply.LogQueryThirdpartInterfaceCount[i].QueryRate = "0%"
		}
		i++
	}

	for i := 0; i < int(reply.Total); i++ {
		for j := i + 1; j < int(reply.Total); j++ {
			if reply.LogQueryThirdpartInterfaceCount[i].ProviderApiId > reply.LogQueryThirdpartInterfaceCount[j].ProviderApiId {
				tmp := reply.LogQueryThirdpartInterfaceCount[i]
				reply.LogQueryThirdpartInterfaceCount[i] = reply.LogQueryThirdpartInterfaceCount[j]
				reply.LogQueryThirdpartInterfaceCount[j] = tmp
			}
		}
	}
	offset := req.PageSize * (req.PageNumber - 1)
	switch {
	case reply.Total-offset <= 0:
		reply.LogQueryThirdpartInterfaceCount = []apis.LogQueryThirdpartInterfaceCount{}
	case reply.Total-offset <= req.PageSize:
		reply.LogQueryThirdpartInterfaceCount = reply.LogQueryThirdpartInterfaceCount[offset:]
	case reply.Total-offset > req.PageSize:
		reply.LogQueryThirdpartInterfaceCount = reply.LogQueryThirdpartInterfaceCount[offset : offset+req.PageSize]
	}
	return nil
}

func logQueryThirdpartInterfaceCount(req *apis.LogQueryThirdpartInterfaceCountReq, reply *apis.LogQueryThirdpartInterfaceCountReply, tabName string) error {
	o := orm.NewOrm()
	where, val := getLogQueryThirdpartInterfaceCountWhere(req)
	sqlStr := "select create_time as date,provider_api_id, provider_api_name,provider_name,count(id) as total ," +
		"sum(state) as success, " +
		"count(IF(state=0,true,null)) as fail," +
		"count(IF(state=1 and code!=1100,true,null)) as query," +
		"count(IF(state=1 and code=1100,true,null)) as un_query," +
		"avg(elapsed) as elapsed  from " + tabName + " %s " +
		"group by provider_api_id order by provider_api_id desc"

	sqlStr = fmt.Sprintf(sqlStr, where)
	_, err := o.Raw(sqlStr, val).QueryRows(&reply.LogQueryThirdpartInterfaceCount)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}
	return nil
}

func LogQueryThirdpartInterfaceCount(ctx context.Context, req *apis.LogQueryThirdpartInterfaceCountReq, reply *apis.LogQueryThirdpartInterfaceCountReply) error {
	if true {
		return LogQueryThirdpartInterfaceCountNew(ctx, req, reply)
	}

	if req.PageSize == 0 {
		req.PageSize = PAGESIZE
	}

	if req.PageNumber == 0 {
		req.PageNumber = 1
	}

	limit := strconv.FormatInt((req.PageNumber-1)*req.PageSize, 10) + "," + strconv.FormatInt(req.PageSize, 10)
	where, val := getLogQueryThirdpartInterfaceCountWhere(req)

	sqlStr := "select create_time as date,provider_api_name,provider_name,count(id) as total ," +
		"sum(state) as success, " +
		"count(IF(state=0,true,null)) as fail," +
		"count(IF(state=1 and code!=1100,true,null)) as query," +
		"count(IF(state=1 and code=1100,true,null)) as un_query," +
		"avg(elapsed) as elapsed  from t_gd_thirdpart_access_log %s " +
		"group by provider_api_id order by provider_api_id desc limit %s"

	sqlStr = fmt.Sprintf(sqlStr, where, limit)

	var ids []int

	count, err := orm.NewOrm().Raw("select id from t_gd_thirdpart_access_log "+where+" group by provider_api_id", val).QueryRows(&ids)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}

	reply.Total = count
	_, err = orm.NewOrm().Raw(sqlStr, val).QueryRows(&reply.LogQueryThirdpartInterfaceCount)
	if err != nil && err != orm.ErrNoRows {
		return errors.DataBaseError
	}

	for index, _ := range reply.LogQueryThirdpartInterfaceCount {
		if reply.LogQueryThirdpartInterfaceCount[index].Total != 0 {
			reply.LogQueryThirdpartInterfaceCount[index].FailRate = strconv.FormatFloat(float64(100*reply.LogQueryThirdpartInterfaceCount[index].Fail)/float64(reply.LogQueryThirdpartInterfaceCount[index].Total), 'f', 2, 64) + "%"
		} else {
			reply.LogQueryThirdpartInterfaceCount[index].FailRate = "0%"
		}
		if reply.LogQueryThirdpartInterfaceCount[index].Success != 0 {
			reply.LogQueryThirdpartInterfaceCount[index].QueryRate = strconv.FormatFloat(float64(100*reply.LogQueryThirdpartInterfaceCount[index].Query)/float64(reply.LogQueryThirdpartInterfaceCount[index].Success), 'f', 2, 64) + "%"
		} else {
			reply.LogQueryThirdpartInterfaceCount[index].QueryRate = "0%"
		}

	}
	reply.PageNumber = req.PageNumber
	reply.PageSize = req.PageSize
	fmt.Println(reply)
	return nil
}
