package access_log

import (
	"context"
	"gd_statistics/apis"
	"gd_statistics/rpc_apis/gd_management"
	"fmt"
	"strconv"
	"strings"
	"time"

	"github.com/astaxie/beego/orm"
)

func getUserCountWhere(req *apis.LogQueryUserAccessCountReq) (w string, s []interface{}) {
	if req.EndTimestamp != 0 && req.StartTimestamp != 0 {
		w += " and " + "timestamp >=" + " ? and timestamp < ?"
		s = append(s, req.StartTimestamp, req.EndTimestamp)
	} else {
		nowTime := time.Now()
		zeroTime := time.Date(nowTime.Year(), nowTime.Month(), nowTime.Day(), 0, 0, 0, 0, nowTime.Location())
		startTimeStamp := zeroTime.Unix()
		endTimeStamp := nowTime.Unix()
		w += " and " + "timestamp >=" + " ? and timestamp <?"
		s = append(s, startTimeStamp, endTimeStamp)
	}
	if req.ApiId != 0 {
		w += " and " + "api_id" + " = ?"
		s = append(s, req.ApiId)
	}
	if req.MerchantId != 0 {
		w += " and " + "merchant_id" + " = ?"
		s = append(s, req.MerchantId)
	}

	if len(s) > 0 {
		//有条件才截取最前面的and
		w = string([]byte(w)[4:])
		w = " where" + w + " and merchant_data_api_id>0"
		return w, s
	} else {
		return " where merchant_data_api_id>0", nil
	}

}

func computeAccessCharge(req *apis.LogQueryUserAccessCountReq,
	timeoutInfo map[string]int, info *apis.LogQueryUserAcessCount,
	countMap map[string]gd_management.UserMerchantCountCode) {

	sqlStr := ""
	mdids := strings.Split(info.MerchantDataApiId, ",")
	for _, mdid := range mdids {
		v, ok := countMap[mdid+fmt.Sprintf("%d", info.ApiId)]
		if ok == false {
			continue
		}
		count := int64(0)
		timeout := 3600 * 24
		if t, ok := timeoutInfo[mdid+fmt.Sprintf("%d", info.ApiId)]; ok && t > 0 {
			timeout = t
		}
		switch {
		case v.CountType == 0 && timeout == 3600*24 && len(mdids) == 1:
			count = info.Total
		case v.CountType == 0 && (timeout != 3600*24 || len(mdids) > 1):

			sqlStr = "select count(id) from t_gd_access_log where merchant_data_api_id = %s and api_id = %d and elapsed < %d and timestamp >= ? and timestamp < ?"
			sqlStr = fmt.Sprintf(sqlStr, mdid, info.ApiId, timeout)
			orm.NewOrm().Raw(sqlStr, req.StartTimestamp, req.EndTimestamp).QueryRow(&count)
		default:
			if v.CountCode == "" {
				count = 0
			} else {
				sqlStr = "select count(id) from t_gd_access_log where merchant_data_api_id = %s and api_id = %d and raw_code in (%s) and timestamp >= ? and timestamp <? and elapsed < %d"
				sqlStr = fmt.Sprintf(sqlStr, mdid, info.ApiId, v.CountCode, timeout)
				orm.NewOrm().Raw(sqlStr, req.StartTimestamp, req.EndTimestamp).QueryRow(&count)
			}
		}
		info.Charge += count
	}

	info.ChargeRate = strconv.FormatFloat(float64(100*info.Charge)/float64(info.Total), 'f', 2, 64) + "%"

}

func LogQueryUserAccessCountNew(ctx context.Context, req *apis.LogQueryUserAccessCountReq, reply *apis.LogQueryUserAccessCountReply) error {
	if req.PageSize == 0 {
		req.PageSize = PAGESIZE
	}

	if req.PageNumber == 0 {
		req.PageNumber = 1
	}

	if req.EndTimestamp == 0 || req.StartTimestamp == 0 {
		nowTime := time.Now()
		zeroTime := time.Date(nowTime.Year(), nowTime.Month(), nowTime.Day(), 0, 0, 0, 0, nowTime.Location())
		req.StartTimestamp = zeroTime.Unix()
		req.EndTimestamp = nowTime.Unix()
	}

	mreq := apis.BiGetReportReq{}
	mreply := apis.BiGetReportReply{}

	mreq.PageNumber = int(req.PageNumber)
	mreq.MerchantId = req.MerchantId
	if req.ApiId != 0 {
		mreq.ApiId = append(mreq.ApiId, req.ApiId)
	}
	for i := req.StartTimestamp; i < req.EndTimestamp; i = i + 24*3600 {
		date := time.Unix(i, 0).Format("2006-01-02")
		mreq.Date = append(mreq.Date, date)
	}
	err := BiGetReportByApi(ctx, &mreq, &mreply)
	if err != nil {
		return err
	}
	reply.PageNumber = int64(mreq.PageNumber)
	reply.PageSize = int64(mreply.PageSize)
	reply.Total = int64(mreply.Total)
	reply.LogQueryUserAcessCount = make([]apis.LogQueryUserAcessCount, len(mreply.Infos))
	for i, v := range mreply.Infos {
		reply.LogQueryUserAcessCount[i].Date = v.Date
		reply.LogQueryUserAcessCount[i].ApiName = v.ApiName
		reply.LogQueryUserAcessCount[i].Total = v.Total
		reply.LogQueryUserAcessCount[i].Query = v.QueryNoRecord
		reply.LogQueryUserAcessCount[i].Success = v.Success
		reply.LogQueryUserAcessCount[i].Valid = v.Valid
		reply.LogQueryUserAcessCount[i].Reuse = v.Reuse
		reply.LogQueryUserAcessCount[i].Charge = v.Charge
		reply.LogQueryUserAcessCount[i].ChargeRate = v.ChargeRate
		reply.LogQueryUserAcessCount[i].Elapsed, _ = strconv.ParseFloat(v.AvgElapsed, 64)

		if v.Total == 0 {
			reply.LogQueryUserAcessCount[i].ValidRate = "0%"
		} else {
			reply.LogQueryUserAcessCount[i].ValidRate = strconv.FormatFloat(float64(100*reply.LogQueryUserAcessCount[i].Valid)/float64(v.Total), 'f', 2, 64) + "%"
		}
		if v.Valid == 0 {
			reply.LogQueryUserAcessCount[i].SuccessRate = "0%"
			reply.LogQueryUserAcessCount[i].ReuseRate = "0%"
			reply.LogQueryUserAcessCount[i].QueryRate = "0%"
		} else {

			reply.LogQueryUserAcessCount[i].SuccessRate = strconv.FormatFloat(float64(100*reply.LogQueryUserAcessCount[i].Success)/float64(v.Valid), 'f', 2, 64) + "%"
			reply.LogQueryUserAcessCount[i].ReuseRate = strconv.FormatFloat(float64(100*v.Reuse)/float64(v.Valid), 'f', 2, 64) + "%"
			reply.LogQueryUserAcessCount[i].QueryRate = strconv.FormatFloat(float64(100*reply.LogQueryUserAcessCount[i].Query)/float64(v.Valid), 'f', 2, 64) + "%"
		}
	}
	return nil
}

func LogQueryUserAccessCount(ctx context.Context, req *apis.LogQueryUserAccessCountReq, reply *apis.LogQueryUserAccessCountReply) error {
	return LogQueryUserAccessCountNew(ctx, req, reply)
}
